-- 
-- (c) Copyright 2008 - 2013 Xilinx, Inc. All rights reserved.
-- 
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
-- 
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
-- 
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
-- 
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES.
-- 
------------------------------------------------------------------------------
-- User entered comments
------------------------------------------------------------------------------
-- This is a self-desigined Memory model developed for EMC Testing purpose
--
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


library unisim;
use unisim.vcomponents.all;
library work;

entity axi_emc_0_exdes_tb is

end entity;

architecture tb of axi_emc_0_exdes_tb is
  COMPONENT axi_emc_0_exdes is
   PORT (
    clk_p       : in std_logic;
    clk_n       : in std_logic;
    reset       : in std_logic;
    atg_done    : out std_logic;
--    dq           : inout std_logic_vector(15 DOWNTO 0);
    dq_i         : in std_logic_vector(15 DOWNTO 0);
    dq_o         : out std_logic_vector(15 DOWNTO 0);
    dq_t         : out std_logic_vector(15 DOWNTO 0);
    a            : out std_logic_vector(31 DOWNTO 0);
    cen          : out std_logic_vector(0 DOWNTO 0);
    oen          : out std_logic_vector(0 DOWNTO 0);
    wen          : out std_logic;
    ben          : out std_logic_vector(1 DOWNTO 0);
    cken         : out std_logic;
    clk          : out std_logic;
    mwait        : in std_logic_vector(0 DOWNTO 0);
    atg_status  : out std_logic_vector(1 DOWNTO 0)
   );
  end COMPONENT;
COMPONENT sram_model is
  generic (
    data_width     : integer := 32;
    addr_width     : integer := 32;
    pipeline_delay : integer := 0;
    parity_val     : integer := 0
  );
  port (
    CLK : in std_logic;
    CKE_N : in std_logic;
    D_IN :   in std_logic_vector (data_width-1 downto 0);
    D_OUT :   out std_logic_vector (data_width-1 downto 0);
    A :   in std_logic_vector (addr_width-1 downto 0);
    CEN : in std_logic;
    OEN : in std_logic;
    WEN : in std_logic;
    PARITY_IN : in std_logic_vector ((data_width/8)-1 downto 0);
    PARITY_OUT : out std_logic_vector ((data_width/8)-1 downto 0);
    BEN : in std_logic_vector ((data_width/8)-1 downto 0)
);
END COMPONENT;

COMPONENT asram_model is
  generic (
    data_width     : integer := 32;
    addr_width     : integer := 32;
    Trc            : integer := 0;
    Taa            : integer := 0;
    Toha           : integer := 0;
    Tace           : integer := 0;
    Tdoe           : integer := 0;
    Thzoe          : integer := 0;
    Thzce          : integer := 0;
    Twc            : integer := 0;
    Tsce           : integer := 0;
    Taw            : integer := 0;
    Tha            : integer := 0;
    Tsa            : integer := 0;
    Tpwe           : integer := 0;
    Tsd            : integer := 0;
    Thd            : integer := 0
  );
  port (
    CLK    : IN STD_LOGIC;
    D_IN   : IN STD_LOGIC_VECTOR (data_width-1 downto 0);
    D_OUT  : OUT STD_LOGIC_VECTOR (data_width-1 downto 0);
    A      : IN STD_LOGIC_VECTOR (addr_width-1 downto 0);
    CEN    : IN STD_LOGIC;
    OEN    : IN STD_LOGIC;
    WEN    : IN STD_LOGIC;
    BEN    : IN STD_LOGIC_VECTOR ((data_width/8)-1 downto 0)
);
end COMPONENT;

signal local_clk_p : std_logic := '0';
signal local_clk_n : std_logic := '1';
signal local_reset : std_logic := '1';
signal local_atg_done : std_logic;
signal local_atg_status : std_logic_vector(1 downto 0);
signal  dq_i        :  std_logic_vector(15 DOWNTO 0);
signal  dq_o        :  std_logic_vector(15 DOWNTO 0);
signal  a        :  std_logic_vector(31 DOWNTO 0);
signal  dq          :  std_logic_vector(15 DOWNTO 0);

signal cen       :  std_logic_vector(0 DOWNTO 0);
signal  oen      :  std_logic_vector(0 DOWNTO 0);
signal  wen      :  std_logic;
signal  ben      :  std_logic_vector(1 DOWNTO 0);
signal  cken     :  std_logic;
signal  clk      :  std_logic;
signal  wr_en    :  std_logic;
-----
begin
-----

wr_en <= not(wen);
--data_m:for j in 0 to 15 generate
--  prh_j : IOBUF
--        port map (
--          IO => dq(j),
--          I => dq_o(j),
--          O => dq_i(j),
--          T => wr_en);
--end generate data_m ;
----------------------------------------------
-- PROCESS_CLK_GEN_P: generate the clock with 100 Mhz
PROCESS_CLK_GEN_P: process
-----
begin
-----
    wait for 2.5 ns;
    local_clk_p <= not local_clk_p;
end process PROCESS_CLK_GEN_P;

------------------------------------------------------------------
----------------------------------------------
-- PROCESS_CLK_GEN_N: generate the clock with 100 Mhz
PROCESS_CLK_GEN_N: process
-----
begin
-----
    wait for 2.5 ns;
    local_clk_n <= not local_clk_n;
end process PROCESS_CLK_GEN_N;

------------------------------------------------------------------
----------------------------------------------
-- PROCESS_RST_GEN: generate the clock with 100 Mhz
PROCESS_RST_GEN: process
-----
begin
-----
    wait for 25 ns;
    local_reset <= '1';
    wait for 70 ns;
    local_reset <= '0';
    wait;
end process PROCESS_RST_GEN;
------------------------------------------------------------------
process (local_atg_done)
-----
begin
-----
        if (local_atg_done'event and local_atg_done = '1') then
          if (local_atg_status(1 downto 0) = "01") then
  report "Memory models used in example design are not accurate and should not be relied upon. They may not support certain combinations of timing parameters" severity NOTE;
            report "Test Completed Successfully" severity FAILURE;
          elsif (local_atg_status(1 downto 0) = "10") then
  report "Memory models used in example design are not accurate and should not be relied upon. They may not support certain combinations of timing parameters" severity NOTE;
            report "Test Failure" severity FAILURE;
          elsif (local_atg_status(1 downto 0) = "11") then
  report "Memory models used in example design are not accurate and should not be relied upon. They may not support certain combinations of timing parameters" severity NOTE;
            report "Test Hanged" severity FAILURE;
          end if;
        end if;
end process;
------------------------------------------------------------------
process
begin
     wait for 10000000 ns;
     report "Test Failed !! Test Timed Out" severity failure;
end process;
------------------------------------------------------------------
MEM_MODEL_0: asram_model
  generic map (
    data_width  => 16, 
    addr_width  => 32,
    Trc         => 15000,
    Taa         => 15002,
    Toha        => 5000,
    Tace        => 12000,
    Tdoe        => 0,
    Thzoe       => 500,
    Thzce       => 5000,
    Twc         => 12000,
    Tsce        => 12000,
    Taw         => 12000,
    Tha         => 0,
    Tsa         => 0,
    Tpwe        => 12000,
    Tsd         => 15000,
    Thd         => 0
  )
  port map (
   CLK     =>     clk,
   D_IN    =>     dq_o(15 downto 0),
   D_OUT   =>     dq_i(15 downto 0),
   A       =>     a,
   CEN     =>     cen(0),
   OEN     =>     oen(0),
   WEN     =>     wen,
   BEN     =>     ben((16/8)-1 downto 0)
  );


EXDES: axi_emc_0_exdes
  port map (
       clk_p       => local_clk_p,
       clk_n       => local_clk_n,
       reset       => local_reset,
       clk            => clk,
       dq_i       =>  dq_i,
       dq_o       =>  dq_o,
       dq_t       => open,
--       dq             => dq, 
       a =>      a,
       cen =>    cen,    
       oen =>    oen,    
       wen =>    wen,    
       ben =>    ben,    
       cken =>   cken,   
       mwait =>   (OTHERS => '0'),
       atg_done    => local_atg_done,
       atg_status  => local_atg_status
  );

end tb;
